package com.ghisler.android.unroot;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;

import org.apache.http.util.EncodingUtils;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import com.ghisler.android.unroot.R;   
 
public class MainActivity extends Activity {

	private String lastGetPartitionInfoError="";
	private PartInfo pi=null;
	Handler mHandler=new Handler();	
	Process process = null;
   	DataOutputStream outputStream = null; 
   	DataInputStream inputStream = null;
	DataInputStream errorStream = null;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		final Button btn=(Button)findViewById(R.id.unrootButton);
		final TextView tv=(TextView)findViewById(R.id.textView1);
		if (btn!=null && tv!=null) {
			File f=new File("/system/xbin/sugote");
			if (f.exists()) {
				btn.setVisibility(View.GONE);
				tv.setText(getString(R.string.supersu_installed));
			} else {
				f=new File("/system/xbin/su");
				boolean exists=f.exists();
				if (f.exists()) {
					f=new File("/system/xbin/daemonsu");
					exists=f.exists();
				}
				if (f.exists()) {
					f=new File("/system/etc/install-recovery.sh");
					exists=f.exists();
				}
				if (!exists) { 
					btn.setVisibility(View.GONE);
					tv.setText(getString(R.string.only_for_towelroot));
				} else {
					Thread thread=new Thread(new Runnable() {
						@Override
						public void run() {
							try {
								Thread.sleep(1000);
							} catch (InterruptedException e) {
							}
				 			pi=getPartitionInfo("/system", true);
				 			mHandler.post(new Runnable() {
								@Override
								public void run() {
									if (pi==null) {
										btn.setVisibility(View.GONE);
										if (lastGetPartitionInfoError.contains("Timeout!"))
											tv.setText(getString(R.string.rooted_but_no_access));
										else
											tv.setText(getString(R.string.device_not_rooted)+lastGetPartitionInfoError);
									} else {
										btn.setEnabled(true);
										btn.setOnClickListener(new OnClickListener() {
											@Override
											public void onClick(View v) {
												removeTowelRoot();					
											}
										});
									}
								}
							});
						}
					});			
					thread.start();
				}
			}
		}
	}

	protected void removeTowelRoot() {
		AlertDialog.Builder dlgAlert  = new AlertDialog.Builder(this);
		dlgAlert.setMessage(getString(R.string.want_unroot));
		dlgAlert.setTitle(getString(R.string.title_warning));
		dlgAlert.setPositiveButton(getString(R.string.btn_ok), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				removeTowelRoot2();
			}
		});
		dlgAlert.setNegativeButton(getString(R.string.btn_cancel), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
			}
		});
		dlgAlert.setCancelable(true);
		dlgAlert.create().show();
	}

	protected void removeTowelRoot2() {
		Button btn=(Button)findViewById(R.id.unrootButton);
		TextView tv=(TextView)findViewById(R.id.textView2);
		if (tv!=null && btn!=null) {
			btn.setEnabled(false);
			String command="";
			if (pi.mountType.toLowerCase().equals("ro")) {
				command=getMountCommand("rw", pi);
				command+="\n";
			}
			command+="rm /system/xbin/su\n";
			command+="rm /system/xbin/daemonsu\n";
			command+="rm /system/etc/install-recovery.sh\n";
			/*if (pi.mountType.toLowerCase().equals("ro")) {  // unfortunately mount as read only always fails -> just reboot
				command+="sync;sync\n";
				command+="cd /\n";
				command+=getMountCommand("ro", pi)+"\n";
			}*/
			command+="echo complete!";
			tv.setText("Running:\n"+command);
			String reply=sendSingleShellRequest(command, true);
			if (reply!=null)
				tv.setText(command+"\n\n"+getString(R.string.result)+":\n"+reply);
		}		
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		int id = item.getItemId();
		if (id == R.id.action_about) {
			AlertDialog.Builder dlgAlert  = new AlertDialog.Builder(this);
			dlgAlert.setMessage(getString(R.string.about_box));
			dlgAlert.setTitle(getString(R.string.app_name));
			dlgAlert.setPositiveButton(getString(R.string.btn_ok), new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which) {
				}
			});
			dlgAlert.setCancelable(true);
			dlgAlert.create().show();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}
	
	public class PartInfo {
		String mountType;
		String device;
		String mountPath;
		String fileSys;
	}
	
	public static String cutlastslash(String dir) {
    	int l=dir.length();
    	if (l>1 && dir.endsWith("/"))
    			dir=dir.substring(0, l-1);
    	return dir;
    }

    public static String strcatslash(String dir) {
    	if (dir.length()>0 && !dir.endsWith("/"))
    		return dir+"/";
    	else
    		return dir;
    }
    
    public String partitionTable=null;

    public PartInfo getPartitionInfo(String curdir, boolean refresh) {
    	lastGetPartitionInfoError="";
    	if (curdir==null)
    		return null;
    	File f=new File(curdir);
    	String curdir2;
    	try {
    		curdir2=f.getCanonicalPath();
    	} catch (Throwable e) {
    		curdir2=null;
    	}
    	if (curdir2!=null)
    		curdir=curdir2;
    	curdir=strcatslash(curdir);
    	if (refresh || partitionTable==null) {
			partitionTable="";
			partitionTable=sendSingleShellRequest("cat /proc/mounts",true);
			if (partitionTable==null)
				return null;
			if (partitionTable.startsWith("*")) {
				lastGetPartitionInfoError=partitionTable.substring(1);
				return null;
			}
    	}
    	try {
    		// partitionTable may be changed by bg thread -> keep copy!
    		String partitionTableLocal=partitionTable;
	    	if (partitionTableLocal!=null && partitionTableLocal.length()>0) {
	    		PartInfo res=null;
	    		int len=partitionTableLocal.length();
	    		int pos=0;
	    		int maxmatchlen=0;
	    		while (pos<len) {
	    			char ch=partitionTableLocal.charAt(pos);
	    			int pos2=partitionTableLocal.indexOf('\n', pos);
	    			if (pos2<0)
	    				pos2=len;
	    			if (ch!='\n') {
	    				String device="";
	    				String mountPath="";
	    				String fileSys="";
	    				String mountType="";
	    				String s2=partitionTableLocal.substring(pos,pos2);
	    				int j=s2.indexOf(' ');
	    				if (j>0) {
	    					device=s2.substring(0,j);
	    					s2=s2.substring(j+1);
	    				}
	    				j=s2.indexOf(' ');
	    				if (j>0) {
	    					mountPath=s2.substring(0,j);
	    					s2=s2.substring(j+1);
	    				}
	    				mountPath=strcatslash(mountPath);
	    				if (mountPath.length()>maxmatchlen) {
	    					if (curdir.startsWith(mountPath)) {
	    	    				j=s2.indexOf(' ');
	    	    				if (j>0) {
	    	    					fileSys=s2.substring(0,j);
	    	    					s2=s2.substring(j+1);
	    	    				}
	    	    				j=s2.indexOf(' ');
	    	    				if (j>0) {
	    	    					mountType=s2.substring(0,j);
	    	    					s2=s2.substring(j+1);
	    	    				}
	    	    				if (mountType.length()>0) {
	        						maxmatchlen=mountPath.length();
	        						if (res==null)
	        							res=new PartInfo();
	        			    		res.device=device;
	        			    		res.fileSys=fileSys;
	        			    		res.mountPath=mountPath;
	        			    		res.mountType=mountType.substring(0,2);
	    	    				}
	    					}
	    				}
	    			}
	    			pos=pos2;
	    			if (pos>=0)
	    				pos++;
	    			else
	    				break;
	    		}
    			return res;
	    	}
    	} catch (Throwable e) {
    	}
    	lastGetPartitionInfoError="Partition not found";
    	return null;
	}

	public String getMountCommand(String mnttype,PartInfo res) {
		String path=cutlastslash(res.mountPath);
		return "mount -o remount,"+mnttype+" -t "+res.fileSys+" "+res.device+" "+path;
		//return "toolbox mount -o remount,"+mnttype+" -t auto "+path;
	}
    
	private void writeUtf8(DataOutputStream stream, String string) throws IOException {
		byte[] cmd=EncodingUtils.getBytes(string, "UTF-8");
        stream.write(cmd); 
	}
	
	static private String specialchars="\r\n \t";
	
	private String sendSingleShellRequest(String command,boolean superUser)
	{
		int waitsteps=50;
		String res="*";
		try {
			Runtime runtime = java.lang.Runtime.getRuntime();
			process = runtime.exec("su");
		} catch (Exception e){
    	    res+="Exception: "+e.getMessage();
		} catch (OutOfMemoryError e){
			res+="Out of memory";
		} catch (Error e) {
			res+="Error: "+e.getMessage();
		}
		if (res.length()>1)
			return res;
		try {
    	   	outputStream = new DataOutputStream(process.getOutputStream()); 
    	   	inputStream = new DataInputStream(process.getInputStream());
			errorStream = new DataInputStream(process.getErrorStream());
			writeUtf8(outputStream,command+"\n");
			outputStream.flush();
			writeUtf8(outputStream,"exit\n");
			outputStream.flush();
		
			byte[] buffer=new byte[8192];
			res="";
			while (true) {
				int avail=inputStream.available();
				if (avail<=0) {      // check whether the thread has terminated!
					try {
						process.exitValue();
						avail=1;
					} catch (Throwable e) {
					}
				}
				if (avail>0) {
					int len=inputStream.read(buffer);
					if (len<0)
						break;
					String line=EncodingUtils.getString(buffer,0,len, "UTF-8"); 
					// delete beginning spaces, tabs or line breaks
					res+=line;
					waitsteps=50;   // always reset when getting data
				} else {
					Thread.sleep(100);
					waitsteps--;
					if (waitsteps<0)
						break;
				}
			}
    		String err="";
			while (true) {
				if (errorStream.available()>0) {
					int len=errorStream.read(buffer);
					if (len<0)
						break;
					String line=EncodingUtils.getString(buffer,0,len, "UTF-8"); 
					err+=line;
				} else {
					break;   // we checked already in data stream
				}
			}
			err=removeFakeErrors(err);
			err=removeLineBreaks(err);
			if (err.length()>0) {
				res+=err;
			}
			// delete beginning spaces, tabs or line breaks
        	while (res.length()>0 && specialchars.indexOf(res.charAt(res.length()-1))>=0)
        		res=res.substring(0,res.length()-1);
        	outputStream.close();
        	inputStream.close();
        	errorStream.close();
        	process.destroy();
		}
		catch (Exception e){
    	    res="*Exception: "+e.getMessage();
		} catch (OutOfMemoryError e){
			res="*Out of memory";
		} catch (Error e) {
			res="*Error: "+e.getMessage();
		}
		try {
			process.destroy();
		} catch (Throwable e) {
		}
		if (waitsteps<0)
			res="*Timeout!\n"+res;
		return res;
	}
	
	private String removeFakeErrors(String str) {
		while (str.startsWith("__bionic_open_tzdata")) {
			int i=str.indexOf('\n');
			if (i>0)
				str=str.substring(i+1);
			else
				str="";
		}
		return str;
	}
	
	private String removeLineBreaks(String str) {
		int l=str.length();
		int l2=l;
		while (l>0 && str.charAt(l-1)=='\n')
			l--;
		int start=0;
		while (start<l && str.charAt(start)=='\n')
			start++;
		if (start>0 || l<l2)
			str=str.substring(start,l);
		return str;
	}

}
